#! Praat script EvoContrast/Erb version/drawBimodal.praat
# Paul Boersma, January 18, 2008

form Draw bimodal evolution
	real Transmission_noise_(Erb) 0.8
	natural Number_of_runs 100000
	choice Decision_strategy 1
		option Optimality Theory
		option Harmonic Grammar
endform
evaluationNoise = 2.0

numberOfGenerations = numberOfSelected ("Matrix")
if numberOfGenerations < 1 or numberOfSelected ("Table") <> 1
	exit Please first select one Table and a number of Matrices.
endif
initialLanguage = selected ("Table")
for generation to numberOfGenerations
	grammar'generation' = selected ("Matrix", generation)
endfor
#
# Consistency check.
#
select initialLanguage
numberOfColumns = Get number of columns
numberOfCategories = numberOfColumns - 1
assert numberOfCategories >= 1
numberOfDivisions = Get number of rows
for generation to numberOfGenerations
	grammar = grammar'generation'
	assert Object_'grammar'. nrow = numberOfCategories + 1
	assert Object_'grammar'. ncol = numberOfDivisions
endfor
select grammar
minimumFrequency = Get lowest x
firstFrequency = Get x of column... 1
maximumFrequency = Get highest x
frequencyStep = Get column distance

history = Create Matrix... history minimumFrequency maximumFrequency numberOfDivisions frequencyStep firstFrequency
... 0 numberOfGenerations numberOfGenerations+1 1 0 0

for generation from 0 to numberOfGenerations
	if generation > 0
		grammar = grammar'generation'
		select grammar
		disharmonies = To Sound (slice)... 1
	endif
	for bin to numberOfDivisions
		sum'bin' = 0
	endfor
	for category to 2
		for run to number_of_runs
			#
			# Production noise: the parent chooses a phonetic frequency.
			#
			if generation = 0
				select initialLanguage
				producedRow = Draw row from distribution... p'category'
				producedFrequency = Get value... producedRow frequency
			else
				select disharmonies
				if decision_strategy$ = "Optimality Theory"
					Formula... max (
					... randomGauss (Object_'grammar' [category, col], evaluationNoise),
					... randomGauss (Object_'grammar' [3, col], evaluationNoise))
				elsif decision_strategy$ = "Harmonic Grammar"
					Formula...
					... randomGauss (Object_'grammar' [category, col], evaluationNoise) +
					... randomGauss (Object_'grammar' [3, col], evaluationNoise)
				else
					exit Unknown decision strategy "'decision_strategy$'".
				endif
				producedFrequency = Get time of minimum... 0 0 None
				; the optimal candidate
			endif
			#
			# Transmission.
			#
			frequency = randomGauss (producedFrequency, transmission_noise)
			frequencyBin = max (min (round ((frequency - firstFrequency) / frequencyStep) + 1, numberOfDivisions), 1)
			sum'frequencyBin' += if category = 1 then -1 else 1 fi
		endfor
	endfor
	select history
	for bin to numberOfDivisions
		Set value... numberOfGenerations-generation+1 bin sum'bin'
	endfor
	if generation > 0
		select disharmonies
		Remove
	endif
endfor
Times
12
Erase all
Select outer viewport... 0 6 0 4
select Matrix history
minimum = Get minimum
maximum = Get maximum
;Formula... min(self,0)
;Formula... log10(self)
;Paint image... min max 1 numberOfGenerations log10(minimum)+1 log10(maximum)
Paint image... minimumFrequency maximumFrequency 0 numberOfGenerations minimum/3 -minimum/3
;Paint image... min max 1 numberOfGenerations -2 2
Marks bottom every... 1 2 yes yes yes
Text bottom... yes Spectral mean (Erb)
for generation from 1 to numberOfGenerations
	One mark right... numberOfGenerations-generation+0.5 no no no 'generation'
endfor
for generation from 1 to numberOfGenerations - 1
	One mark right... numberOfGenerations-generation no no yes
endfor
Text right... yes Generations
Draw inner box
Select outer viewport... 0 6 0.3 4