#! Praat script EvoContrast/Erb version/drawEvolution.praat
# Paul Boersma, January 26, 2008

form Draw evolution
	real Transmission_noise_(Erb) 0.8
	natural Number_of_runs 100000
	choice Decision_strategy 1
		option Optimality Theory
		option Harmonic Grammar
		option Maximum Entropy
endform
evaluationNoise = 2.0

numberOfGenerations = numberOfSelected ("Matrix")
if numberOfGenerations < 1 or numberOfSelected ("Table") <> 1
	exit Please first select one Table and a number of Matrices.
endif
initialLanguage = selected ("Table")
for generation to numberOfGenerations
	grammar'generation' = selected ("Matrix", generation)
endfor
#
# Consistency check.
#
select initialLanguage
numberOfColumns = Get number of columns
numberOfCategories = numberOfColumns - 1
assert numberOfCategories >= 1
numberOfDivisions = Get number of rows
for generation to numberOfGenerations
	grammar = grammar'generation'
	assert Object_'grammar'. nrow = numberOfCategories + 1
	assert Object_'grammar'. ncol = numberOfDivisions
endfor
select grammar
minimumFrequency = Get lowest x
maximumFrequency = Get highest x
clearinfo
#
# Compute the means and standard deviations of the categories of the initial language.
#
print ; Generation 0:
select initialLanguage
for category to numberOfCategories
	meanProbability = Get mean... p'category'
	Append product column... frequency p'category' temp
	meanTemp = Get mean... temp
	Formula... temp self * self ["frequency"]
	meanTemp2 = Get mean... temp
	Remove column... temp
	meanFrequency = meanTemp / meanProbability
	meanFrequency2 = meanTemp2 / meanProbability
	stdevFrequency = sqrt (meanFrequency2 - meanFrequency ^ 2 + transmission_noise ^ 2)
	print    'meanFrequency:6' ('stdevFrequency:6')
	mean'category'_0 = meanFrequency
	stdev'category'_0 = stdevFrequency
endfor
printline
#
# Compute the means and standard deviations of the categories of the new generations.
#
for generation from 0 to numberOfGenerations
	print ; Generation 'generation':
	if generation > 0
		grammar = grammar'generation'
		select grammar
		disharmonies = To Sound (slice)... 1
	endif
	for category to numberOfCategories
		sumFrequency = 0.0
		sumFrequency2 = 0.0
		for run to number_of_runs
			#
			# Production noise: the parent chooses a phonetic frequency.
			#
			if generation = 0
				select initialLanguage
				producedRow = Draw row from distribution... p'category'
				producedFrequency = Get value... producedRow frequency
			else
				select disharmonies
				if decision_strategy$ = "Optimality Theory"
					Formula... max (
					... randomGauss (Object_'grammar' [category, col], evaluationNoise),
					... randomGauss (Object_'grammar' [numberOfCategories + 1, col], evaluationNoise))
					producedFrequency = Get time of minimum... 0 0 None
				elsif decision_strategy$ = "Harmonic Grammar"
					Formula...
					... randomGauss (Object_'grammar' [category, col], evaluationNoise) +
					... randomGauss (Object_'grammar' [numberOfCategories + 1, col], evaluationNoise)
					producedFrequency = Get time of minimum... 0 0 None
				elsif decision_strategy$ = "Maximum Entropy"
					# Harmony:
					Formula... Object_'grammar' [category, col] + Object_'grammar' [numberOfCategories + 1, col]
					# Relative probability:
					Formula... self [col - 1] + exp (- self)
					# Choose:
					sum = Get maximum... 0 0 None
					pick = randomUniform (0, sum)
					Reverse
					Formula... ( self [col + 1] > pick ) <> ( self [col] > pick )
					Reverse
					maximum = Get maximum... 0 0 None
					assert maximum = 1.0   ; 'sum' 'pick'
					producedFrequency = Get time of maximum... 0 0 None
				else
					exit Unknown decision strategy "'decision_strategy$'".
				endif
				; the optimal candidate
			endif
			#
			# Transmission.
			#
			frequency = randomGauss (producedFrequency, transmission_noise)
			sumFrequency += frequency
			sumFrequency2 += frequency ^ 2
		endfor
		meanFrequency = sumFrequency / number_of_runs
		meanFrequency2 = sumFrequency2 / number_of_runs
		stdevFrequency = sqrt (meanFrequency2 - meanFrequency ^ 2)
		print    'meanFrequency:6' ('stdevFrequency:6')
		mean'category'_'generation' = meanFrequency
		stdev'category'_'generation' = stdevFrequency
	endfor
	if generation > 0
		select disharmonies
		Remove
	endif
	printline
endfor
Times
12
Erase all
Select outer viewport... 0 6 0 4
Axes... minimumFrequency maximumFrequency 0 numberOfGenerations
Marks bottom every... 1 2 yes yes yes
Text bottom... yes Spectral mean (Erb)
for generation from 1 to numberOfGenerations
	One mark right... numberOfGenerations-generation+0.5 no no no 'generation'
endfor
for generation from 1 to numberOfGenerations - 1
	One mark right... numberOfGenerations-generation no no yes
endfor
Text right... yes Generations
Create simple Matrix...  grey 2 2*numberOfGenerations+2 0
for category to numberOfCategories
	select Matrix grey
	for generation from 0 to numberOfGenerations
		mean = mean'category'_'generation'
		stdev = stdev'category'_'generation'
		Set value... 1 1+generation mean-stdev
		Set value... 2 1+generation numberOfGenerations-generation
		Set value... 1 2*numberOfGenerations+2-generation mean+stdev
		Set value... 2 2*numberOfGenerations+2-generation numberOfGenerations-generation
	endfor
	To Polygon
	Paint... 0.8 minimumFrequency maximumFrequency 0 numberOfGenerations
	Remove
endfor
Create simple Matrix... black 2 numberOfGenerations+1 0
Line width... 5
for category to numberOfCategories
	select Matrix black
	for generation from 0 to numberOfGenerations
		mean = mean'category'_'generation'
		Set value... 1 1+generation mean
		Set value... 2 1+generation numberOfGenerations-generation
	endfor
	To Polygon
	Draw... minimumFrequency maximumFrequency 0 numberOfGenerations
	Remove
endfor
select Matrix grey
plus Matrix black
Remove
Line width... 1
Draw inner box
Select outer viewport... 0 6 0.3 4