# Praat script evoContrast/simulateBimodal.praat
# Paul Boersma, January 17, 2008

form Simulate the evolution of contrast
	real Minimum_frequency_(Erb) 20.0
	real Maximum_frequency_(Erb) 36.0
	positive Frequency_step_(Erb) 0.1
	real Mean1a 25.0
	real Mean1b 31.0
	real Mean2 28.0
	positive Initial_production_noise 0.8
	positive Transmission_noise 0.8
	positive Neighbourhood 0.5
	natural Number_of_trials 2000000
	natural Number_of_generations 20
	choice Decision_strategy 1
		option Optimality Theory
		option Harmonic Grammar
endform
evaluationNoise = 2.0
plasticity = 0.01
numberOfDivisions = round ((maximum_frequency - minimum_frequency) / frequency_step) + 1
Times
12
initialLanguage = Create Table with column names... language numberOfDivisions frequency
Formula... frequency minimum_frequency + (row - 1) * frequency_step
factor = -0.5 / initial_production_noise ^ 2
Append column... p1
Formula... p1 0.25 * exp (factor * (self ["frequency"] - mean1a) ^ 2) + 0.75 * exp (factor * (self ["frequency"] - mean1b) ^ 2)
Append column... p2
Formula... p2 exp (factor * (self ["frequency"] - mean2) ^ 2)
for generation to number_of_generations
	childGrammar = Create Matrix... grammar'generation' minimum_frequency maximum_frequency numberOfDivisions frequency_step minimum_frequency
	... 1 3 3 1 1 100
	disharmonies = To Sound (slice)... 1
	for trial to number_of_trials
		intendedCategory = randomInteger (1, 2)
		if generation = 1
			select initialLanguage
			producedRow = Draw row from distribution... p'intendedCategory'
			producedFrequency = Get value... producedRow frequency
		else
			select disharmonies
			if decision_strategy$ = "Optimality Theory"
				Formula... max (
				... randomGauss (Object_'parentGrammar' [intendedCategory, col], evaluationNoise),
				... randomGauss (Object_'parentGrammar' [3, col], evaluationNoise))
			elsif decision_strategy$ = "Harmonic Grammar"
				Formula...
				... randomGauss (Object_'parentGrammar' [intendedCategory, col], evaluationNoise) +
				... randomGauss (Object_'parentGrammar' [3, col], evaluationNoise)
			else
				exit Unknown decision strategy "'decision_strategy$'".
			endif
			producedFrequency = Get time of minimum... 0 0 None
			; the optimal candidate
		endif
		receivedFrequency = randomGauss (producedFrequency, transmission_noise)
		ivalue = max (min (round ((receivedFrequency - minimum_frequency) / frequency_step) + 1, numberOfDivisions), 1)   ; determine the two violated constraints
		perceivedCategory = imin (
		... randomGauss (Object_'childGrammar' [1, ivalue], evaluationNoise),
		... randomGauss (Object_'childGrammar' [2, ivalue], evaluationNoise))   ; the optimal candidate
		if intendedCategory <> perceivedCategory
			select childGrammar
			factor = -0.5 / neighbourhood ^ 2
			Formula... if row = intendedCategory then self - plasticity * exp (factor * (x - receivedFrequency) ^ 2)
			... else if row = perceivedCategory then self + plasticity * exp (factor * (x - receivedFrequency) ^ 2)
			... else self fi fi
		endif
	endfor
	select childGrammar
	Formula... if row = 3 then 91 + 13 * ((x - (minimum_frequency + maximum_frequency) / 2) / ((maximum_frequency - minimum_frequency) / 2)) ^ 2 else self fi   ; articulatory constraints
	execute drawGrammar.praat 1
	parentGrammar = childGrammar
	select disharmonies
	Remove
endfor
